@@ -11,7 +11,7 @@ from gatenlp.annotation_set import AnnotationSet
 from gatenlp.annotation import Annotation
 from gatenlp.offsetmapper import OffsetMapper, OFFSET_TYPE_PYTHON, OFFSET_TYPE_JAVA
 from gatenlp.features import Features
-from gatenlp.utils import in_notebook
+from gatenlp.utils import in_notebook, in_colab
 from gatenlp.changelog import ChangeLog
 
 from gatenlp.changelog_consts import (
@@ -224,10 +224,12 @@ class Document:
                         ann.features.clear()
                         ann.features.update(features)
                     elif handle_existing_anns == ADDANN_ADD_NEW_FEATURES:
-                        fns = ann.feature_names()
+                        fns = ann.features.names()
                         for f in features.keys():
                             if f not in fns:
                                 ann.features[f] = features[f]
+                    elif handle_existing_anns == ADDANN_IGNORE:
+                        pass
 
             elif cmd == ACTION_CLEAR_ANNS:
                 assert sname is not None
@@ -338,9 +340,16 @@ class Document:
 
     @text.setter
     def text(self, value: str) -> None:
-        """Set the text of the document. This is only possible as long as it has not been set
+        """
+        Set the text of the document. This is only possible as long as it has not been set
         yet, after that, the text is immutable.
 
+        IMPORTANT: it is possible to add arbitrary annotations to a document which does not have any
+        text. This is meant to allow handling of annotation-only representations.
+        However, if the text is set after annotations have been added, annotation offsets are not
+        checked and it is possible to thus create an invalid document where annotations refer to
+        text ranges that do not exist!
+
         Args:
           value: the text for the document
           value: str:
@@ -402,7 +411,7 @@ class Document:
         if isinstance(span, AnnotationSet):
             return self.text[span.start():span.end()]
         if hasattr(span, "start") and hasattr(span, "end"):
-            return self.text[span.start, span.end]
+            return self.text[span.start:span.end]
         return self.text[span]
 
     def annset(self, name: str = "") -> AnnotationSet:
@@ -812,31 +821,74 @@ class Document:
         but instead initializes the JS in the notebook unless gatenlp.init_notebook()
         has bee called already.
         """
-        return self._notebook_show()
+        if in_colab():
+            return self._show_colab(display=False)
+        else:
+            return self._show_notebook(display=False)
 
     # TODO: maybe allow manual selection of how to show the document, e.g. also by
     # writing to a tmp file and browsing in a browser, or pprint etc.
-    def show(self, htmlid=None, annsets=None):
+    def show(self, to=None, htmlid=None, annsets=None, doc_style=None):
         """
-        Show the document in a Jupyter notebook. This allows to assign a specific htmlid so
+        Show the document, possibly in a Jupyter notebook. This allows to assign a specific htmlid so
         the generated HTML can be directly styled afterwards.
-        This directly sends the rendered document to the cell (no display/HTML necessary).
+        This directly sends the rendered document to the cell (no display/HTML necessary) if
+        the destination is a notebook.
 
         Args:
+            to: if None, try to guess if this is called from within a notebook and if yes, which kind.
+                Otherwise, explicitly specify where to show the document to, one of "console", "jupyter",
+                "colab".
             htmlid: the HTML id prefix to use for classes and element ids.
             annsets: if not None, a list of annotation set/type specifications.
                 Each element is either
                 the name of a set to fully include, or a tuple with the name of the set as
                 the first element
                 and with a single type name or a list of type names as the second element
-
+            doc_style: if not None, use this as the style for the document text box
         """
+        if to == "colab":
+            self._show_colab(htmlid=htmlid, display=True, annsets=annsets, doc_style=doc_style)
+            return
+        elif to == "jupyter":
+            self._show_notebook(htmlid=htmlid, display=True, annsets=annsets, doc_style=doc_style)
+            return
+        elif to == "console":
+            return self.__str__()
+        elif to is not None:
+            raise Exception(f"Not a valid value for parameter to: {to}. Use one of console, jupyter, colab")
         if in_notebook():
-            self._notebook_show(htmlid=htmlid, display=True, annsets=annsets)
+            if in_colab():
+                self._show_colab(htmlid=htmlid, display=True, annsets=annsets, doc_style=doc_style)
+                return
+            else:
+                self._show_notebook(htmlid=htmlid, display=True, annsets=annsets, doc_style=doc_style)
+                return
         else:
             return self.__str__()
 
-    def _notebook_show(self, htmlid=None, display=False, annsets=None):
+    def _show_colab(self, htmlid=None, display=False, annsets=None, doc_style=None):
+        from gatenlp.serialization.default import HtmlAnnViewerSerializer, JS_GATENLP_URL, JS_JQUERY_URL
+        from IPython.display import display_html, Javascript
+        from IPython.display import display as i_display
+
+        i_display(Javascript(url=JS_JQUERY_URL))
+        i_display(Javascript(url=JS_GATENLP_URL))
+        html = self.save_mem(
+            fmt="html-ann-viewer",
+            notebook=True,
+            add_js=False,
+            offline=True,
+            htmlid=htmlid,
+            annsets=annsets,
+            doc_style=doc_style,
+        )
+        if display:
+            display_html(html, raw=True)
+        else:
+            return html
+
+    def _show_notebook(self, htmlid=None, display=False, annsets=None, doc_style=None):
         from gatenlp.gatenlpconfig import gatenlpconfig
         from gatenlp.serialization.default import HtmlAnnViewerSerializer
         from IPython.display import display_html
@@ -851,12 +903,36 @@ class Document:
             offline=True,
             htmlid=htmlid,
             annsets=annsets,
+            doc_style=doc_style,
         )
         if display:
             display_html(html, raw=True)
         else:
             return html
 
+    def attach(self, annset, name, check=True):
+        """
+        Attach a detached set to the document. This should get used with caution and is mainly
+        intended for use inside the gatenlp library to allow for fast incremental creation of
+        new documents and document sets. The set can only be added if a set with the given name
+        does not yet exist at all.
+
+        Args:
+            annset: the annotation set to attach
+            name: the name for the annotation set
+            check: if False, prevent any checking. WARNING: this may create an inconsistent/illegal document!
+        """
+        if name in self._annotation_sets:
+            raise Exception(f"Cannot attach set, a set with the name {name} already exists")
+        if check:
+            # check if the offsets are consistent with the document
+            l = len(self)
+            for ann in annset._annotations.values():
+                if ann.end > l:
+                    raise Exception(f"Cannot attach set, annotation beyond text end: {ann}")
+        self._annotation_sets[name] = annset
+        annset._owner_doc = self
+
 
 class MultiDocument(Document):
     """
