@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 """
-Module for interacting with a Java GATE process, running API commands on it and
-exchanging data with it.
+Module for interacting with a Java GATE process.
 """
 
 import sys
@@ -16,6 +15,7 @@ import signal
 import glob
 import json
 from gatenlp.annotation_set import AnnotationSet
+from gatenlp.urlfileutils import is_url
 
 # NOTE: we delay importing py4j to the class initializer. This allows us to make GateWorker available via gatenlp
 # but does not force everyone to actually have py4j installed if they do not use the GateWorker
@@ -283,37 +283,34 @@ class GateWorker:
                the worker.
         debug: show debug messages (default: False)
         """
-        self.logger = init_logger(__name__)
+        if debug:
+            self.logger = init_logger("GateWorker", lvl="DEBUG")
+        else:
+            self.logger = init_logger("GateWorker")
 
         from py4j.java_gateway import JavaGateway, GatewayParameters
 
-        self.gatehome = gatehome
-        self.port = port
-        self.host = host
-        self.start = start
-        self.gatehome = gatehome
-        self.platform = platform
-        self.gateprocess = None
-        self.gateway = None
-        self.worker = None
-        self.closed = False
-        self.keep = keep
-        self.debug = debug
-        self.log_actions = log_actions
+        self._gatehome = gatehome
+        self._port = port
+        self._host = host
+        self._platform = platform
+        self._gateprocess = None
+        self._gateway = None
+        self._closed = False
         if use_auth_token:
             if not auth_token:
-                self.auth_token = secrets.token_urlsafe(20)
+                self._auth_token = secrets.token_urlsafe(20)
             else:
-                self.auth_token = auth_token
+                self._auth_token = auth_token
         else:
-            self.auth_token = ""
+            self._auth_token = ""
         if gatehome is None and start:
             gatehome = os.environ.get("GATE_HOME")
             if gatehome is None:
                 raise Exception(
                     "Parameter gatehome is None and environment var GATE_HOME not set"
                 )
-            self.gatehome = gatehome
+            self._gatehome = gatehome
         if start:
             # make sure we find the jar we need
             # logger.info("DEBUG: file location: {}".format(__file__))
@@ -340,14 +337,16 @@ class GateWorker:
                 cmdandparms.append("1")
             else:
                 cmdandparms.append("0")
-            os.environ["GATENLP_WORKER_TOKEN_" + str(self.port)] = self.auth_token
+            os.environ["GATENLP_WORKER_TOKEN_" + str(self.port)] = self._auth_token
             cmd = " ".join(cmdandparms)
             self.logger.debug(f"Running command: {cmd}")
             subproc = subprocess.Popen(
                 cmdandparms, stderr=subprocess.PIPE, bufsize=0, encoding="utf-8"
             )
-            self.gateprocess = subproc
+            self._gateprocess = subproc
+            haveerror = False
             while True:
+                # NOTE: the following line can block when the subprocess ends with an error
                 line = subproc.stderr.readline()
                 if line == "":
                     break
@@ -355,34 +354,89 @@ class GateWorker:
                 if line == "PythonWorkerRunner.java: server start OK":
                     break
                 if line == "PythonWorkerRunner.java: server start NOT OK":
-                    raise Exception("Could not start server, giving up")
+                    haveerror = True
+                    # if we get an error we continue to loop through stderr until we get end of stream (line=="")
+                    # and only raise the exception after the loop.
+                if line.startswith("Java GatenlpWorker ENDING"):
+                    break
                 print(line, file=sys.stderr)
+            if haveerror:
+                raise Exception("Error when starting server")
             atexit.register(self.close)
-        self.gateway = JavaGateway(
-            gateway_parameters=GatewayParameters(port=port, auth_token=self.auth_token)
+        self._gateway = JavaGateway(
+            gateway_parameters=GatewayParameters(port=port, auth_token=self._auth_token)
         )
-        self.jvm = self.gateway.jvm
-        self.worker = self.gateway.entry_point
-        self.gate_version = self.jvm.gate.Main.version
-        self.gate_build = self.jvm.gate.Main.build
-        self.worker_version = self.worker.pluginVersion()
-        self.worker_build = self.worker.pluginBuild()
 
-    @staticmethod
-    def download():
-        """
-        Download GATE libraries into a standard location so we can run the GATE worker even if GATE_HOME
-        is not set.
+    @property
+    def jvm(self):
+        return self.gateway.jvm
 
-        NOTE YET IMPLEMENTED.
+    @property
+    def worker(self):
+        return self.gateway.entry_point
+
+    @property
+    def gate_version(self):
         """
-        # TODO: this should use the command and bootstrapping jar in gate-downloader:
-        # copy the whole directory into the standard per-user config directory for the system
-        # run the command
-        # use the generated gate.classpath as for a compiled local git repo
-        # NOTE: should change error message if GATE_HOME is not set to hint at this! (option --downlaod for the script)
-        # NOTE: add to documentation
-        raise Exception("Not yet implemented")
+        Return the GATE version of the connected GATE process.
+        """
+        return self.jvm.gate.Main.version
+
+    @property
+    def gate_build(self):
+        """
+        Return the GATE build id of the connected GATE process.
+        """
+        return self.jvm.gate.Main.build
+
+    @property
+    def worker_version(self):
+        return self.worker.pluginVersion()
+
+    @property
+    def worker_build(self):
+        return self.worker.pluginBuild()
+
+    @property
+    def gatehome(self):
+        return self._gatehome
+
+    @property
+    def port(self):
+        return self._port
+
+    @property
+    def host(self):
+        return self._host
+
+    @property
+    def platform(self):
+        return self._platform
+
+    @property
+    def gateprocess(self):
+        return self._gateprocess
+
+    @property
+    def gateway(self):
+        return self._gateway
+
+    # @staticmethod
+    # def download():
+    #     """
+    #     Download GATE libraries into a standard location so we can run the GATE worker even if GATE_HOME
+    #     is not set.
+    #
+    #     NOTE YET IMPLEMENTED.
+    #     """
+    #     # TODO: this should use the command and bootstrapping jar in gate-downloader:
+    #     # copy the whole directory into the standard per-user config directory for the system
+    #     # run the command
+    #     # use the generated gate.classpath as for a compiled local git repo
+    #     # NOTE: should change error message if GATE_HOME is not set to hint at this!
+    #     # (option --downlaod for the script)
+    #     # NOTE: add to documentation
+    #     raise Exception("Not yet implemented")
 
     def close(self):
         """
@@ -390,14 +444,20 @@ class GateWorker:
         Otherwise we can still close it if it was started by the workerrunner, not the Lr
         Note: if it was started by us, it was started via the workerrunner.
         """
-        if not self.closed and self.worker.isClosable():
-            self.closed = True
+        if not self._closed and self.worker.isClosable():
+            self._closed = True
             self.gateway.shutdown()
             if self.gateprocess is not None:
                 for line in self.gateprocess.stderr:
                     print(line, file=sys.stderr, end="")
                 self.gateprocess.wait()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, _exptype, _value, _traceback):
+        self.close()
+
     def log_actions(self, onoff):
         """
         Switch logging actions at the worker on or off.
@@ -435,7 +495,7 @@ class GateWorker:
         """
         if mimetype is None:
             mimetype = ""
-        self.worker.saveDocumentToFile(path, mimetype)
+        self.worker.saveDocumentToFile(gdoc, path, mimetype)
 
     def gdoc2pdoc(self, gdoc):
         """
@@ -462,8 +522,8 @@ class GateWorker:
         Returns:
             handle to GATE document
         """
-        json = pdoc.save_mem(fmt="bdocjs", annsets=annsets)
-        return self.worker.getDocument4BdocJson(json)
+        jsondata = pdoc.save_mem(fmt="bdocjs", annsets=annsets)
+        return self.worker.getDocument4BdocJson(jsondata)
 
     def gdocanns2pdoc(self, gdoc, pdoc, annsets=None, replace=False):
         """
@@ -580,24 +640,6 @@ class GateWorker:
         """
         return self.worker.findMavenPlugin(group, artifact)
 
-    def gate_build(self):
-        """
-        Return the short commit id of the Java GATE we are connected to.
-
-        Returns:
-            short commit id string
-        """
-        return self.worker.gate_build()
-
-    def gate_version(self):
-        """
-        Return the version string of the Java GATE we are connected to.
-
-        Returns:
-            version string
-        """
-        return self.worker.gate_version()
-
     def getBdocJson(self, gdoc):
         """
         Return the Bdoc JSON serialization of a Java GATE document as string.
@@ -641,7 +683,7 @@ class GateWorker:
         Returns:
             handle to the Java GATE document
         """
-        return self.worker.getDocument4BdocJson
+        return self.worker.getDocument4BdocJson(bdocjson)
 
     def getDocument4Name(self, name):
         """
@@ -720,7 +762,6 @@ class GateWorker:
         """
         return self.worker.getResources4Name(name)
 
-
     def getResources4NameClass(self, name, clazz):
         """
         Return a (possibly empty) list of all Java GATE resources with the given name and class name.
@@ -783,6 +824,18 @@ class GateWorker:
         """
         return self.worker.loadPipelineFromFile(filename)
 
+    def loadPipelineFromUri(self, uri):
+        """
+        Load a pipeline/controller from the given uri into Java GATE and return a CorpusController handle to it.
+
+        Args:
+            uri: the uri of the pipeline file
+
+        Returns:
+            a CorpusController handle to the loaded Java GATE pipeline
+        """
+        return self.worker.loadPipelineFromUri(uri)
+
     def loadPipelineFromPlugin(self, group, artifact, path):
         """
         Load a prepared pipeline from the given loaded GATE Mave plugin into Java GATE and return
@@ -825,7 +878,6 @@ class GateWorker:
         """
         return self.worker.pluginBuild()
 
-
     def pluginVersion(self):
         """
         Return the version string of the Python plugin on the Java GATE side.
@@ -844,7 +896,6 @@ class GateWorker:
         """
         self.worker.print2err(message)
 
-
     def print2out(self, message):
         """
         Output the given message to System.out on the Java GATE side.
@@ -971,19 +1022,15 @@ class GateWorker:
 
 
 class GateWorkerAnnotator(Annotator):
-    # TODO: something that starts a gate worker when created, loads pipeline in Java GATE,
-    # sends over document
-    # or document and selection of annotation sets/annotation types, runs pipeline,
-    # and then fetches one or more annotation sets and updates the local document with them.
     # TODO: parameter to influence how exceptions are handled
     def __init__(
         self,
         pipeline,
-        gatehome=None,
-        port=25333,
+        gateworker,
         annsets_send=None,
         annsets_receive=None,
         replace_anns=False,
+        update_document=False,
     ):
         """
         Create a GateWorker annotator.
@@ -1003,43 +1050,46 @@ class GateWorkerAnnotator(Annotator):
         Example:
 
             ```python
-            pipeline = GateWorkerAnnotator("annie.xgapp")
+            pipeline = GateWorkerAnnotator("annie.xgapp", GateWorker())
             for idx, doc in enumerate(mycorpus):
                 corpus[idx] = pipeline(doc)
             ```
 
         Args:
             pipeline: the path to a Java GATE pipeline to load into the GATE worker
-            gatehome: the gate home directory to use, if not set, uses environment variable GATE_HOME
-            port: the port to use (25333)
+            gateworker: the gate home directory to use, if not set, uses environment variable GATE_HOME
             annsets_send: a list of either annotation set names, or tuples where the first element
                 is the name of an annotation set and the second element is either the name of a type
                 or a list of type names. If not None, only the sets/types specified are sent to Java GATE.
                 If an empty list is specified, no annotations are sent at all.
-            annsets_receive: same format as annsets_send to specify which annotation sets/types are
+            annsets_receive: this only works if update_document is True: same format as annsets_send to specify
+                which annotation sets/types are
                 sent back to Python after the document has been processed on the Java side.
-            replace_anns: if True and an annotation is received which already exists (same set and annotation id)
-              then the existing annotation is replaced (if offsets and type are also same, only the features are
-              replaced). If False, all received annotations are added which may change their annotation id.
+            replace_anns: this is only relevant if update_document is True: if True and an annotation is received
+                which already exists (same set and annotation id)
+                then the existing annotation is replaced (if offsets and type are also same, only the features are
+                replaced). If False, all received annotations are added which may change their annotation id.
+            update_document: if True, then existing annotations in the gatenlp document are kept and the annotations
+                received from Java GATE are added. In this case, other changes to the document, e.g. the document
+                text or document features are not applied to the current python document.
+                If False, the existing document is completely replaced with what gets
+                received from Java GATE.
         """
         self.pipeline = pipeline
         self.annsets_send = annsets_send
         self.annsets_receive = annsets_receive
         self.replace_anns = replace_anns
-        self.gs = GateWorker(port=port, start=True, gatehome=gatehome)
-        self.controller = self.gs.worker.loadPipelineFromFile(self.pipeline)
-        self.corpus = self.gs.worker.newCorpus()
+        self.update_document = update_document
+        self.gw = gateworker
+        isurl, ext = is_url(pipeline)
+        if isurl:
+            self.controller = self.gw.worker.loadPipelineFromUri(ext)
+        else:
+            self.controller = self.gw.worker.loadPipelineFromFile(ext)
+        self.corpus = self.gw.worker.newCorpus()
         self.controller.setCorpus(self.corpus)
         self.controller.setControllerCallbacksEnabled(False)
 
-    def close(self):
-        """
-        Shut down the GateWorker used by this annotator.
-
-        After calling this, the GateWorkerAnnotator instance cannot be used any more.
-        """
-        self.gs.close()
-
     def start(self):
         """
         Invoke the controller execution started method on the GATE controller.
@@ -1052,7 +1102,7 @@ class GateWorkerAnnotator(Annotator):
         """
         self.controller.invokeControllerExecutionFinished()
 
-    def __call__(self, doc, **kwargs):
+    def __call__(self, doc, **_kwargs):
         """
         Run the GATE controller on the given document.
 
@@ -1072,10 +1122,13 @@ class GateWorkerAnnotator(Annotator):
             tmpdoc = doc.copy(annsets=self.annsets_send)
         else:
             tmpdoc = doc
-        gdoc = self.gs.pdoc2gdoc(tmpdoc)
-        self.gs.worker.run4Document(self.controller, gdoc)
-        self.gs.gdocanns2pdoc(gdoc, doc, annsets=self.annsets_receive, replace=self.replace_anns)
-        self.gs.del_resource(gdoc)
+        gdoc = self.gw.pdoc2gdoc(tmpdoc)
+        self.gw.worker.run4Document(self.controller, gdoc)
+        if self.update_document:
+            self.gw.gdocanns2pdoc(gdoc, doc, annsets=self.annsets_receive, replace=self.replace_anns)
+        else:
+            doc = self.gw.gdoc2pdoc(gdoc)
+        self.gw.del_resource(gdoc)
         return doc
 
 
@@ -1121,7 +1174,7 @@ def main():
     ap.add_argument("--debug", action="store_true", help="Show debug messages")
     args = ap.parse_args()
     if args.download:
-        GateWorker.download()
+        raise Exception("--download not implemented yet ")
     else:
         start_gate_worker(
             port=args.port,
